/*
 * keydb.h
 *
 * Header for the strings db toolkit.
 *
 * This file defines standard structures and prototypes for use by other
 * programs in the package.  For details of implementation for the functions,
 * see the comments in strdb.c for each function.
 *
 * 0.3.0, March 23, 1997.  Keys.
 * 0.4.0, March 24, 1997.  No mods.
 * 0.5.0, June 17, 1997.  Added minimum length storage (changes minor version).
 *
 * Copyright (c) 1996 Amelia A. Lewis
 * Distributed under GPL.  See COPYING.
 *
 * $Id: keydb.h 13 2004-03-27 20:21:34Z amyzing $
 */

#ifndef sdb_KEYDB_H
#define sdb_KEYDB_H

#include <sys/param.h>
#include <sys/types.h>

/*
 * The header for a key file (version 0.3.0+).  It shares many characteristics
 * of the index file header, and must always point to an index file.
 * The magic is in different offsets.
 */
typedef struct
{
    char sdb_major;
    char sdb_minor;
    char sdb_patch;
    char sdb_field;			/* the field keyed */
    char sdb_magic[4];
    unsigned short sdb_total;		/* total number of keys */
    unsigned char sdb_discthresh;	/* discard threshold, % records */
    unsigned char sdb_minlen;		/* minimum key length stored */
    time_t sdb_mtime;			/* mod time for the index file */
    char sdb_indexfile[MAXPATHLEN];	/* absolute path name of index file */
} sdb_key_head;

/*
 * The key structure.  Note that it is *not* stored in this fashion on
 * disk.  A function, sdb_key_write, is provided in keystr to write the
 * structure to file; strdb.o contains the read function.  Data is stored
 * into the file, after the header, as:
 * key length (one byte).
 * key (no null terminator).
 * number of index entries containing the key (unsigned short).
 * index numbers (unsigned shorts).
 * This means that there is an effective limitation to the number of
 * records in a given file of USHRT_MAX.  If necessary, this can be
 * changed, but 64K records (on all the systems I know) is probably
 * sufficient.
 * The index numbers are used as parameters to sdb_pointer, for instance.
 */
typedef struct
{
    char *key;				/* the key itself, lowercase alnum */
    unsigned short *indices;		/* index numbers of pointers to matching records */
    unsigned short nindices;		/* number of pointers to indices */
} sdb_key;

/*
 * The remainder of the file is function prototypes.  For explanations, see
 * the implementations.
 */

int
sdb_key_write(sdb_key *, FILE *);

char *
sdb_key_read(sdb_key *, FILE *);

void
sdb_key_free(sdb_key *);

int
sdb_key_match(sdb_key *, char *);

#endif /* sdb_KEYDB_H */
