/*
 * sigd.c
 *
 * This application is part of the strings db package.  It is a daemon
 * (but can be run standalone) which creates a random signature for users
 * based on the contents of a file of signatures and a configuration file.
 *
 * version history
 * 0.1, April 27, 1997, for strdb 0.4.3.
 *
 * Copyright (c) 1997 Amelia A. Lewis.  All rights reserved.
 * Distributed under GPL.  See ../COPYING.
 *
 * $Id: sigd.c 31 2005-03-18 04:34:19Z amyzing $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "../lib/strdb.h"

/*
Okay.  Dual-mode application, with symlink.  Called as sigd, it runs as a
daemon, setting up a unix domain socket, where it listens for requests.
On startup, it reads the system-wide configuration file, which establishes
a system-wide signatures db.
Request is in a simple form: user, use/don't use system-wide file, optional
user-configuration file.  If no file, use standard file; if not exist, use
system defaults.  This request "registers" the user.
Then, at requested intervals (not less than system minimum), the daemon
updates the signature file of the particular user, iff the user is logged
in.  So, check for logged in, then check ownership of personal file to
be accessed (group/other-writable not allowed), mail errors to user.
Compose a signature, write it to user's .signature.  Go to sleep.
The daemon needs to keep a list of registered users, with the system-use
flag, and if there are more parameters to be stored, then put them in
a file indexed by user, with an empty field indicating default.

The non-daemon mode is also "dual" mode, because if it can't find the socket,
it just tries to run by itself.  Or else it doesn't.  I dunno.  If run
in combination with the daemon, it should be run at logon to tell the
daemon that you're logged in, and run at logoff to tell the daemon not
to use up more resources.  Then the daemon does all the work.

Otherwise, if it's called with no daemon running, it has no access to system-
wide files, only to what's in the user's home directory, and it won't do
repeated updates, it will only change .signature once.

Configuration file:
maxlines
maxbytes
update interval
personal signatures file
invariant part/format code (repeatable)
    This can include GECOS fields and other fields from /etc/passwd, which
    can also be overridden.
fullname
title (repeatable)
email address (repeatable)
snail address (repeatable)
phone (repeatable)
*/

void
usage(char *calledas)
{
}

void
getargs(int ac, char *av[])
{
}

/*
 * return error if config files not found
 */
int
init_config(char *cfn)
{
    return 0;
}

void
gather_files(void)
{
}

int
main(int argc, char *argv[])
{
    unsigned which, max;
    char linein[80], buf[1024];
    FILE *sig;

    srandom(time(NULL) + getpid());
    /* call infostr on (initially hard-coded) index file */
    fgets(linein, 80, popen("infostr -n ~/signatures.index", "r"));
    max = strtoul(linein, NULL, 10);
    printf("Max: %d\n", max);
    /* get a random value in the range */
    which = 1 + (int) ((max * 1.0) * random() / (RAND_MAX + 1.0));
    printf("Which: %d\n", which);
    /* popen getstr to retrieve the record selected */
    sprintf(linein, "getstr -t %d ~/signatures.index", which);
    fread((char *)buf, sizeof(char), 1024, popen(linein, "r"));
    buf[strlen(buf) - 3] = '\0';
    sig = popen("cat ->~/.signature", "w");
    fwrite(buf, sizeof(char), strlen(buf), sig);
    pclose(sig);
    return 0;
}
