/*
 * edstr.c
 *
 * Part of the strings db toolkit.
 *
 * edstr is a simple utility.  It opens the $EDITOR or $VISUAL program
 * on a tempfile.  On return, it parses the tempfile for correctness, if
 * necessary prints an error message, and reopens the file, until it gets
 * something that it likes.  It then checks the datafile associated with
 * the index file, making certain that that file ends with a delimiter.
 * If not, it fixes it.  It cats the new record on the end, then calls
 * idxstr with the update flag.
 *
 * 0.4.1, April 11, 1997.  First pass.
 * 0.5.0, June 17, 1997.  Getting closer
 *
 * Copyright (c) 1996 Amy A. Lewis
 * Distributed under GPL.  See COPYING.
 *
 * This is fuxxored.
 *
 * $Id: edstr.c 13 2004-03-27 20:21:34Z amyzing $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/stat.h>
#include "../lib/strdb.h"
#include "../lib/maintdb.h"
#include "../lib/errdb.h"

char index_fn[MAXPATHLEN],
     editor[MAXPATHLEN] = "",
     *progname;

FILE *indexf,
     *dataf,
     *tempf;

int babble_flag = 0;
int gibberish_flag = 0;
int order_flag = 0;
int update_flag = 0;

unsigned char n_recs = 0;
unsigned long recnums[255];

sdb_recptr asrp;

char sigch;
char *format_string = NULL;

void
usage(char *ca)
{
    sdb_version(ca, NULL);
    fprintf(stderr, "Usage:\n\t%s [-FhuUvV] [-e ed] [-f strg] idxfile [recnum [...recnum]]\n\n", ca);
    fprintf(stderr, "\t-e ed\tUse editor \"ed\" to edit the file.\
\n	-F	Force use of an outdated index file.\
\n	-f strg	Parse the record according to the format in 'strg'.\
\n	-h	Print this usage message.\
\n	-o	(with -f) Fields must appear in given order.\
\n	-u	Update the index file.\
\n	-U	Update the index file, dump it, and update again.\
\n	-v	Be verbose.\
\n	-V	Report version and exit.\
\nidxfile:	Name of index file to be used.\
\nrecnum:		Index number of record to be edited.\n");
    exit(1);
}

void
getargs(int ac, char *av[])
{
    int i;
    
    if (strrchr(av[0], '/'))
        progname = strdup(strrchr(av[0], '/'));
    else
        progname = strdup(av[0]);
    while ((i = getopt(ac, av, "e:f:FhouUvV")) != EOF)
        switch (i)
        {
            case 'e':
                if (*optarg)
                    strcpy(editor, optarg);
                break;
            case 'f':
                if (*optarg)
                    format_string = strdup(optarg);
                break;
            case 'F':
                gibberish_flag++;
                break;
            case 'o':
                order_flag++;
                break;
            case 'U':
                update_flag++;
            case 'u':
                update_flag++;
                break;
            case 'v':
                babble_flag++;
                break;
            case 'V':
                sdb_version(progname, NULL);
                exit(0);
                break;
            case 'h':
            case '?':
            case ':':
            default:
                usage(progname);
                break;
        }
    
    av += optind;
    if (*av)
        strcpy(index_fn, *av);
    else
        usage(progname);
    if (*(av + 1))
    {
        while (*++av)
        {
            recnums[n_recs] = strtoul(*av, NULL, 10);
            n_recs++;
        }
    }
}

void
getfiles(void)
{
    tempf = tmpfile();
/*        if (!tempf)
            tempf = fopen(temp_fn, "w+");
*/    if (!tempf)
    {
        perror("Unable to create temporary file");
        exit(sdb_ENOENT);
    }

    /* get the index file */
    /* set sigch! */
}

/*
 * Just plays with the passed in format string, to make it more or less
 * sensible, or something on that order.
 */
void
massage_format(void)
{
    fprintf(stderr, "Format strings not implemented.  Shoot the programmer.\n");
}

/*
 * Call the user's favorite editor on the file containing the record.
 * When finished, read the record into the global sdb_recptr asrp.
 */
void
do_edit(void)
{
    struct stat tempstat;
    char cmdstr[MAXPATHLEN];
    
    /* set up a temporary file. */
    if (asrp.data)
        fwrite((char *)asrp.data, sizeof(char), strlen(asrp.data), tempf);
    
    /* find the favored editor */
    if (editor[0])
    {
       if (!stat(editor, &tempstat))
       {
           if (!(tempstat.st_mode & S_IXOTH))
               editor[0] = '\0';
       }
       else
           editor[0] = '\0';
    }
    if (!editor[0])
    {
        if (getenv("EDITOR"))
            strcpy(editor, getenv("EDITOR"));
        else if (getenv("VISUAL"))
            strcpy(editor, getenv("VISUAL"));
        else if (!stat("/bin/vi", &tempstat))
        {
            if (tempstat.st_mode & S_IXOTH)
                strcpy(editor, "/bin/vi");
        }
        else if (!stat("/bin/ed", &tempstat))
        {
            if (tempstat.st_mode & S_IXOTH)
                strcpy(editor, "/bin/ed");
        }
    }
    /* open the editor */
/*    sprintf(cmdstr, "%s %s", editor, temp_fn);
    system(cmdstr);
*/    
    /* returning from that, stat the tempfile, set size */
    fstat(fileno(tempf), &tempstat);
    asrp.data_size = tempstat.st_size;
    asrp.data_os = 0;
    
    fseek(tempf, 0L, SEEK_SET);
    sdb_recfree(&asrp);
    sdb_recalloc(tempf, &asrp);
}

/*
 * basically a wrapper for sdb_parserec
 */
int
parse_record(void)
{
    linerr *parserr = NULL;
    
    if (!asrp.data || (strlen(asrp.data) < 4))
        return 0;

    /* parserr is NULL on success */
    parserr = sdb_parserec(asrp.data, sigch, format_string);
    if (parserr)
    {
        fprintf(stderr, "Line %d: %s", parserr->lineno, sdb_maint_err[parserr->errcode]);
        fprintf(stderr, "Press [Enter] to continue\n");
        getchar();
        free(parserr);
        return 0;
    }
    return 1;
}

/*
 * sig_handler
 *	If this is called, then the user has mashed the quit or interrupt
 * key, to signal that he don't want to keep going and adding the record.
 * All we need to do is to clean up properly.
 */
void
sig_handler(int signo)
{
    fclose(indexf);
    fclose(dataf);
    fclose(tempf);
    exit(signo);
}

int
main(int argc, char *argv[])
{
    struct sigaction ssact;
    char cmdstr[MAXPATHLEN + 10];
    
    asrp.data_os = asrp.data_size = 0;
    asrp.data = NULL;
    
    getargs(argc, argv);
    getfiles();
    
    if (format_string)
        massage_format();

    /* install signal handler */
    sigemptyset(&ssact.sa_mask);
    sigaddset(&ssact.sa_mask, SIGINT);
    sigaddset(&ssact.sa_mask, SIGQUIT);
    ssact.sa_flags = 0;
    ssact.sa_handler = sig_handler;
    sigaction(SIGINT, &ssact, NULL);
    sigaction(SIGQUIT, &ssact, NULL);
    
    do
        do_edit();
    while (!parse_record);
    
    /* reset signal handling */
    ssact.sa_handler = SIG_DFL;
    sigemptyset(&ssact.sa_mask);
    sigaction(SIGINT, &ssact, NULL);
    sigaction(SIGQUIT, &ssact, NULL);

    fseek(dataf, 0L, SEEK_END);
    if (!sdb_check_df(dataf, sigch))
    {
        fprintf(dataf, "\n%c%c\n", sigch, sigch);
        fseek(dataf, 0L, SEEK_END);
    }
    /* append the record */
    asrp.data_os = ftell(dataf);
    fwrite((char *)asrp.data, sizeof(char), strlen(asrp.data), dataf);
    
    /* regenerate the index file */
    sprintf(cmdstr, "idxstr -u %s", index_fn);
    if (babble_flag)
        fprintf(stderr, "Regenerating index.\n");
    system(cmdstr);
    
    return 0;
}
